@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018, 2019 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -64,7 +65,7 @@ USA.
   (set! source-nodes
 	(map (lambda (filename)
 	       (let ((node (make/source-node filename)))
-		 (hash-table/put! source-hash filename node)
+		 (hash-table-set! source-hash filename node)
 		 node))
 	     source-filenames))
   (initialize/syntax-dependencies!)
@@ -100,7 +101,7 @@ USA.
   (%make/source-node filename (->pathname filename)))
 
 (define (filename->source-node filename)
-  (let ((node (hash-table/get source-hash filename #f)))
+  (let ((node (hash-table-ref/default source-hash filename #f)))
     (if (not node)
 	(error "Unknown source file:" filename))
     node))
@@ -147,14 +148,14 @@ USA.
   (for-each (lambda (node)
 	      (set-source-node/dependencies!
 	       node
-	       (list-transform-negative (source-node/backward-closure node)
-		 (lambda (node*)
-		   (memq node (source-node/backward-closure node*)))))
+	       (remove (lambda (node*)
+			 (memq node (source-node/backward-closure node*)))
+		       (source-node/backward-closure node)))
 	      (set-source-node/dependents!
 	       node
-	       (list-transform-negative (source-node/forward-closure node)
-		 (lambda (node*)
-		   (memq node (source-node/forward-closure node*))))))
+	       (remove (lambda (node*)
+			 (memq node (source-node/forward-closure node*)))
+		       (source-node/forward-closure node))))
 	    nodes))
 
 (define (compute-ranks! nodes)
@@ -189,11 +190,14 @@ USA.
   (for-each
    (lambda (node)
      (let ((modification-time
-	    (let ((source (modification-time node "scm"))
-		  (binary (modification-time node "bin")))
-	      (if (not source)
-		  (error "Missing source file" (source-node/filename node)))
-	      (and binary (< source binary) binary))))
+	    (receive (scm bin spec)
+		     (sf/pathname-defaulting (source-node/pathname node) #f #f)
+	      spec
+	      (let ((source (file-modification-time scm))
+		    (binary (file-modification-time bin)))
+		(if (not source)
+		    (error "Missing source file" (source-node/filename node)))
+		(and binary (< source binary) binary)))))
      (set-source-node/modification-time! node modification-time)
      (if (not modification-time)
 	 (write-notification-line
@@ -207,23 +211,24 @@ USA.
 	 (lambda (node)
 	   (let ((time (source-node/modification-time node)))
 	     (if (and time
-		      (there-exists? (source-node/dependencies node)
-			(lambda (node*)
-			  (let ((newer?
-				 (let ((time*
-					(source-node/modification-time node*)))
-				   (or (not time*)
-				       (> time* time)))))
-			    (if newer?
-				(write-notification-line
-				 (lambda (port)
-				   (write-string "Binary file " port)
-				   (write (source-node/filename node) port)
-				   (write-string " newer than dependency "
-						 port)
-				   (write (source-node/filename node*)
-					  port))))
-			    newer?))))
+		      (any (lambda (node*)
+			     (let ((newer?
+				    (let ((time*
+					   (source-node/modification-time
+					    node*)))
+				      (or (not time*)
+					  (> time* time)))))
+			       (if newer?
+				   (write-notification-line
+				    (lambda (port)
+				      (write-string "Binary file " port)
+				      (write (source-node/filename node) port)
+				      (write-string " newer than dependency "
+						    port)
+				      (write (source-node/filename node*)
+					     port))))
+			       newer?))
+			   (source-node/dependencies node)))
 		 (set-source-node/modification-time! node #f))))
 	 source-nodes)
 	(for-each
@@ -248,14 +253,20 @@ USA.
   (write-notification-line
    (lambda (port)
      (write-string "Begin pass 1:" port)))
-  (for-each (lambda (node)
-	      (if (not (source-node/modification-time node))
-		  (source-node/syntax! node)))
-	    source-nodes/by-rank)
-  (if (there-exists? source-nodes/by-rank
-	(lambda (node)
-	  (and (not (source-node/modification-time node))
-	       (source-node/circular? node))))
+  (bind-condition-handler (list condition-type:simple-warning)
+      (lambda (condition)
+	(if (string=? (access-condition condition 'MESSAGE)
+		      "Missing externs file:")
+	    (muffle-warning)))
+    (lambda ()
+      (for-each (lambda (node)
+		  (if (not (source-node/modification-time node))
+		      (source-node/syntax! node)))
+		source-nodes/by-rank)))
+  (if (any (lambda (node)
+	     (and (not (source-node/modification-time node))
+		  (source-node/circular? node)))
+	   source-nodes/by-rank)
       (begin
 	(write-notification-line
 	 (lambda (port)
@@ -306,13 +317,8 @@ USA.
      ((if compiler:enable-integration-declarations?
 	  identity-procedure
 	  (lambda (declarations)
-	    (list-transform-negative declarations
-	      integration-declaration?)))
+	    (remove integration-declaration? declarations)))
       (source-node/declarations node)))))
-
-(define (modification-time node type)
-  (file-modification-time
-   (pathname-new-type (source-node/pathname node) type)))
 
 ;;;; Syntax dependencies
 
@@ -333,9 +339,9 @@ USA.
 			      "refctx" "rvalue" "scode" "sets" "subprb"
 			      "switch" "utils")
 	     (filename/append "back"
-			      "asmmac" "bittop" "bitutl" "insseq" "lapgn1"
-			      "lapgn2" "lapgn3" "linear" "regmap" "symtab"
-			      "syntax")
+			      "asmmac" "bittop" "bitutl" "checks" "insseq"
+			      "lapgn1" "lapgn2" "lapgn3" "linear" "regmap"
+			      "symtab" "syntax")
 	     (filename/append "machines/x86-64"
 			      "dassm1" "insmac" "lapopt" "machin" "rgspcm"
 			      "rulrew")
@@ -408,7 +414,7 @@ USA.
 		  (filename/append "machines/x86-64" "insutl")))
 	 (lapgen-body
 	  (append
-	   (filename/append "back" "lapgn1" "lapgn2" "syntax")
+	   (filename/append "back" "checks" "lapgn1" "lapgn2" "syntax")
 	   (filename/append "machines/x86-64"
 			    "rules1" "rules2" "rules3" "rules4"
 			    "rulfix" "rulflo")))
@@ -543,6 +549,9 @@ USA.
     (file-dependency/integration/join (append assembler-base assembler-body)
 				      assembler-base)
 
+    (define-integration-dependencies "back" "checks" "base" "cfg1" "cfg2")
+    (define-integration-dependencies "back" "checks" "rtlbase"
+      "rtlcfg" "rtlobj" "rtlty1")
     (define-integration-dependencies "back" "lapgn1" "base"
       "cfg1" "cfg2" "utils")
     (define-integration-dependencies "back" "lapgn1" "rtlbase"
