@@ -13,12 +13,12 @@
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or (at
 # your option) any later version.
-# 
+#
 # MIT/GNU Scheme is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # General Public License for more details.
-# 
+#
 # You should have received a copy of the GNU General Public License
 # along with MIT/GNU Scheme; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301,
@@ -28,14 +28,15 @@
 
 set -e
 : ${MIT_SCHEME_EXE=mit-scheme}
-${MIT_SCHEME_EXE} --batch-mode <<\EOF
+: ${MIT_SCHEME_FLAGS=}
+${MIT_SCHEME_EXE} --batch-mode --no-init-file ${MIT_SCHEME_FLAGS} <<\EOF
 
 (begin
   (load "edwin.sf")
   (load "edwin.cbf"))
 EOF
 suffix=`echo "(display (microcode-id/operating-system-suffix))" \
-	| ${MIT_SCHEME_EXE} --batch-mode`
+	| ${MIT_SCHEME_EXE} --batch-mode --no-init-file`
 report=edwin-$suffix.crf
 if [ -s "$report" ]; then
     awk "{printf \"$report:%d: %s\\n\",NR,\$0}" $report
